"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import Enum
from typing import Literal

from .. import cimpl as cimpl

class ScramMechanism(Enum):
    UNKNOWN: Literal["cimpl.SCRAM_MECHANISM_UNKNOWN"]
    SCRAM_SHA_256: Literal["cimpl.SCRAM_MECHANISM_SHA_256"]
    SCRAM_SHA_512: Literal["cimpl.SCRAM_MECHANISM_SHA_512"]
    def __lt__(self, other: "ScramMechanism") -> bool: ...

class ScramCredentialInfo:
    def __init__(self, mechanism: "ScramMechanism", iterations: int) -> None: ...

class UserScramCredentialsDescription:
    def __init__(self, user: str, scram_credential_infos: list["ScramCredentialInfo"]) -> None: ...

class UserScramCredentialAlteration:
    def __init__(self, user: str) -> None: ...

class UserScramCredentialUpsertion(UserScramCredentialAlteration):
    def __init__(
        self, user: str, scram_credential_info: "ScramCredentialInfo", password: bytes, salt: bytes | None = None
    ) -> None: ...

class UserScramCredentialDeletion(UserScramCredentialAlteration):
    def __init__(self, user: str, mechanism: "ScramMechanism") -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from ._model import ConsumerGroupState as ConsumerGroupState
from ._model import ConsumerGroupTopicPartitions as ConsumerGroupTopicPartitions
from ._model import Node as Node
from .cimpl import OFFSET_BEGINNING as OFFSET_BEGINNING
from .cimpl import OFFSET_END as OFFSET_END
from .cimpl import OFFSET_INVALID as OFFSET_INVALID
from .cimpl import OFFSET_STORED as OFFSET_STORED
from .cimpl import TIMESTAMP_CREATE_TIME as TIMESTAMP_CREATE_TIME
from .cimpl import TIMESTAMP_LOG_APPEND_TIME as TIMESTAMP_LOG_APPEND_TIME
from .cimpl import TIMESTAMP_NOT_AVAILABLE as TIMESTAMP_NOT_AVAILABLE
from .cimpl import Consumer as Consumer
from .cimpl import Message as Message
from .cimpl import Producer as Producer
from .cimpl import TopicPartition as TopicPartition
from .cimpl import libversion as libversion
from .deserializing_consumer import DeserializingConsumer as DeserializingConsumer
from .error import KafkaError as KafkaError
from .error import KafkaException as KafkaException
from .serializing_producer import SerializingProducer as SerializingProducer

__all__ = [
    "admin",  # type: ignore
    "Consumer",
    "KafkaError",
    "KafkaException",
    "kafkatest",  # type: ignore
    "libversion",
    "Message",
    "OFFSET_BEGINNING",
    "OFFSET_END",
    "OFFSET_INVALID",
    "OFFSET_STORED",
    "Producer",
    "DeserializingConsumer",
    "SerializingProducer",
    "TIMESTAMP_CREATE_TIME",
    "TIMESTAMP_LOG_APPEND_TIME",
    "TIMESTAMP_NOT_AVAILABLE",
    "TopicPartition",
    "Node",
    "ConsumerGroupTopicPartitions",
    "ConsumerGroupState",
]

__version__: str

class ThrottleEvent:
    def __init__(self, broker_name: str, broker_id: int, throttle_time: float) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from pathlib import Path
from typing import TYPE_CHECKING

from ..avro.error import ClientError as ClientError

if TYPE_CHECKING:
    """Helping users who installed avro package can get type hints"""
    try:
        # pypi/conda library
        from avro.schema import Schema  # type: ignore # this cannot be resolved if avro isn't installed
    except ImportError:
        # standard library
        from typing import Any as Schema  # type: ignore

def loads(schema_str: str) -> "Schema": ...
def load(fp: str | Path) -> "Schema": ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from .conversion_util import ConversionUtil as ConversionUtil
from .validation_util import ValidationUtil as ValidationUtil



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from .cimpl import Consumer as _ConsumerImpl
from .cimpl import Message
from .error import ConsumeError as ConsumeError
from .error import KeyDeserializationError as KeyDeserializationError
from .error import ValueDeserializationError as ValueDeserializationError
from .serialization import MessageField as MessageField
from .serialization import SerializationContext as SerializationContext

class DeserializingConsumer(_ConsumerImpl):
    def __init__(self, conf: dict) -> None: ...
    def poll(self, timeout: int | float = -1) -> Message | None: ...  # type: ignore[override]
    def consume(self, num_messages: int = -1, timeout: int | float = -1) -> None: ...  # type: ignore[override]



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import ClassVar

class SerializerError(Exception):
    message: ClassVar[str]

    def __init__(self, message: str) -> None: ...

class KeySerializerError(SerializerError): ...
class ValueSerializerError(SerializerError): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.

Python client implementations of the official Kafka tests/kafkatest clients.
"""
from __future__ import annotations



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""

# This module is going to be deprecated

# standard library
from typing import Any, Callable

from ..avro.cached_schema_registry_client import CachedSchemaRegistryClient as CachedSchemaRegistryClient
from ..avro.error import ClientError as ClientError
from ..avro.load import load as load
from ..avro.load import loads as loads
from ..avro.serializer import KeySerializerError as KeySerializerError
from ..avro.serializer import SerializerError as SerializerError
from ..avro.serializer import ValueSerializerError as ValueSerializerError
from ..avro.serializer.message_serializer import MessageSerializer as MessageSerializer
from ..cimpl import Consumer as Consumer
from ..cimpl import Message
from ..cimpl import Producer as Producer

class AvroProducer(Producer):
    def __init__(
        self,
        config,
        default_key_schema: Any = None,
        default_value_schema: Any = None,
        schema_registry: Any = None,
        **kwargs
    ) -> None: ...
    def produce(
        self,
        topic: str,
        value: str | bytes | None = None,
        key: str | bytes | None = None,
        partition: int | None = None,
        on_delivery: Callable | None = None,
        timestamp: int = 0,
        headers: dict[str, str | bytes] | list[tuple[str, str | bytes]] | None = None,
    ) -> None: ...

class AvroConsumer(Consumer):
    def __init__(
        self,
        config,
        schema_registry: Any = None,
        reader_key_schema: Any = None,
        reader_value_schema: Any = None,
        **kwargs
    ) -> None: ...
    def poll(self, timeout: int | float | None = None) -> Message: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from io import BytesIO
from typing import Callable

from ..serialization import Deserializer as Deserializer
from ..serialization import SerializationContext
from ..serialization import SerializationError as SerializationError
from ..serialization import Serializer as Serializer
from . import Schema as Schema
from . import topic_subject_name_strategy as topic_subject_name_strategy
from .schema_registry_client import SchemaRegistryClient

class _ContextStringIO(BytesIO):
    def __enter__(self) -> _ContextStringIO: ...
    def __exit__(self, *args): ...

class AvroSerializer(Serializer):
    def __init__(
        self,
        schema_registry_client: SchemaRegistryClient,
        schema_str: str | Schema,
        to_dict: Callable[[object], SerializationContext] | None = None,
        conf: dict | None = None,
    ) -> None: ...
    def __call__(self, obj: object, ctx: SerializationContext) -> None: ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631

class AvroDeserializer(Deserializer):
    def __init__(
        self,
        schema_registry_client: SchemaRegistryClient,
        schema_str: str | Schema | None = None,
        from_dict: Callable[[dict], SerializationContext] | None = None,
        return_record_name: bool = False,
    ) -> None: ...
    def __call__(self, data: bytes, ctx: SerializationContext) -> None: ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import EnumMeta

class ConversionUtil:
    @staticmethod
    def convert_to_enum(val: object, enum_clazz: "EnumMeta") -> "EnumMeta": ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from io import BytesIO
from typing import Callable

from ..schema_registry import Schema as Schema
from ..schema_registry import topic_subject_name_strategy as topic_subject_name_strategy
from ..schema_registry.schema_registry_client import SchemaRegistryClient
from ..serialization import Deserializer as Deserializer
from ..serialization import SerializationContext
from ..serialization import SerializationError as SerializationError
from ..serialization import Serializer as Serializer

class _ContextStringIO(BytesIO):
    def __enter__(self) -> _ContextStringIO: ...
    def __exit__(self, *args): ...

class JSONSerializer(Serializer):
    def __init__(
        self,
        schema_str: str | Schema,
        schema_registry_client: SchemaRegistryClient,
        to_dict: Callable[[object], SerializationContext] | None = None,
        conf: dict | None = None,
    ) -> None: ...
    def __call__(self, obj: object, ctx: SerializationContext): ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631

class JSONDeserializer(Deserializer):
    def __init__(
        self,
        schema_str: str | Schema,
        from_dict: Callable[[dict], SerializationContext] | None = ...,
        schema_registry_client: SchemaRegistryClient | None = None,
    ) -> None: ...
    def __call__(self, data: bytes, ctx: SerializationContext): ...  # type: ignore # issue: https://github.com/confluentinc/confluent-kafka-python/issues/1631



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
import io

from ..serialization import SerializationContext
from ..serialization import SerializationError as SerializationError
from . import reference_subject_name_strategy as reference_subject_name_strategy
from . import topic_subject_name_strategy as topic_subject_name_strategy
from .schema_registry_client import Schema as Schema
from .schema_registry_client import SchemaReference as SchemaReference
from .schema_registry_client import SchemaRegistryClient

class _ContextStringIO(io.BytesIO):
    def __enter__(self) -> _ContextStringIO: ...
    def __exit__(self, *args): ...

class ProtobufSerializer:
    def __init__(self, msg_type: "GeneratedProtocolMessageType", schema_registry_client: SchemaRegistryClient, conf: dict | None = None) -> None: ...  # type: ignore
    def __call__(self, message: "Message", ctx: SerializationContext): ...  # type: ignore

class ProtobufDeserializer:
    def __init__(self, message_type: "MessageType", conf: dict | None = None) -> None: ...  # type: ignore
    def __call__(self, data: bytes, ctx: SerializationContext): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import ClassVar

# pypi/conda library
from fastavro.schema import SchemaParseException as SchemaParseException
from fastavro.schema import UnknownType as UnknownType

class SchemaRegistryError(Exception):
    http_status_code: ClassVar[int]
    error_code: ClassVar[int]
    error_message: ClassVar[str]

    UNKNOWN: int = -1

    def __init__(self, http_status_code, error_code, error_message) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import Enum
from typing import Literal

class ResourceType(Enum):
    """
    UNKNOWN = RESOURCE_UNKNOWN  #: Resource type is not known or not set.
    ANY = RESOURCE_ANY  #: Match any resource, used for lookups.
    TOPIC = RESOURCE_TOPIC  #: Topic resource. Resource name is topic name.
    GROUP = RESOURCE_GROUP  #: Group resource. Resource name is group.id.
    BROKER = RESOURCE_BROKER  #: Broker resource. Resource name is broker id.
    """

    UNKNOWN: Literal["RESOURCE_UNKNOWN"]
    ANY: Literal["RESOURCE_ANY"]
    TOPIC: Literal["RESOURCE_TOPIC"]
    GROUP: Literal["RESOURCE_GROUP"]
    BROKER: Literal["RESOURCE_BROKER"]
    def __lt__(self, other: "ResourceType") -> bool: ...

class ResourcePatternType(Enum):
    """
    UNKNOWN = RESOURCE_PATTERN_UNKNOWN  #: Resource pattern type is not known or not set.
    ANY = RESOURCE_PATTERN_ANY  #: Match any resource, used for lookups.
    MATCH = RESOURCE_PATTERN_MATCH  #: Match: will perform pattern matching
    LITERAL = RESOURCE_PATTERN_LITERAL  #: Literal: A literal resource name
    PREFIXED = RESOURCE_PATTERN_PREFIXED  #: Prefixed: A prefixed resource name
    """

    UNKNOWN: Literal["RESOURCE_PATTERN_UNKNOWN"]
    ANY: Literal["RESOURCE_PATTERN_ANY"]
    MATCH: Literal["RESOURCE_PATTERN_MATCH"]
    LITERAL: Literal["RESOURCE_PATTERN_LITERAL"]
    PREFIXED: Literal["RESOURCE_PATTERN_PREFIXED"]
    def __lt__(self, other: "ResourcePatternType") -> bool: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from asyncio import Future
from typing import Any

from .._model import ConsumerGroupState, ConsumerGroupTopicPartitions
from ..cimpl import CONFIG_SOURCE_DEFAULT_CONFIG as CONFIG_SOURCE_DEFAULT_CONFIG
from ..cimpl import CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG as CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG
from ..cimpl import CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG as CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG
from ..cimpl import CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG as CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG
from ..cimpl import CONFIG_SOURCE_STATIC_BROKER_CONFIG as CONFIG_SOURCE_STATIC_BROKER_CONFIG
from ..cimpl import CONFIG_SOURCE_UNKNOWN_CONFIG as CONFIG_SOURCE_UNKNOWN_CONFIG
from ..cimpl import RESOURCE_ANY as RESOURCE_ANY
from ..cimpl import RESOURCE_BROKER as RESOURCE_BROKER
from ..cimpl import RESOURCE_GROUP as RESOURCE_GROUP
from ..cimpl import RESOURCE_TOPIC as RESOURCE_TOPIC
from ..cimpl import RESOURCE_UNKNOWN as RESOURCE_UNKNOWN
from ..cimpl import KafkaException as KafkaException
from ..cimpl import NewPartitions as NewPartitions
from ..cimpl import NewTopic as NewTopic
from ..cimpl import _AdminClientImpl as _AdminClientImpl
from ._acl import AclBinding as AclBinding
from ._acl import AclBindingFilter as AclBindingFilter
from ._acl import AclOperation as AclOperation
from ._acl import AclPermissionType as AclPermissionType
from ._config import AlterConfigOpType as AlterConfigOpType
from ._config import ConfigEntry as ConfigEntry
from ._config import ConfigResource as ConfigResource
from ._config import ConfigSource as ConfigSource
from ._group import ConsumerGroupDescription as ConsumerGroupDescription
from ._group import ConsumerGroupListing as ConsumerGroupListing
from ._group import ListConsumerGroupsResult as ListConsumerGroupsResult
from ._group import MemberAssignment as MemberAssignment
from ._group import MemberDescription as MemberDescription
from ._metadata import BrokerMetadata as BrokerMetadata
from ._metadata import ClusterMetadata as ClusterMetadata
from ._metadata import GroupMember as GroupMember
from ._metadata import GroupMetadata as GroupMetadata
from ._metadata import PartitionMetadata as PartitionMetadata
from ._metadata import TopicMetadata as TopicMetadata
from ._resource import ResourcePatternType as ResourcePatternType
from ._resource import ResourceType as ResourceType
from ._scram import ScramCredentialInfo as ScramCredentialInfo
from ._scram import ScramMechanism as ScramMechanism
from ._scram import UserScramCredentialAlteration as UserScramCredentialAlteration
from ._scram import UserScramCredentialDeletion as UserScramCredentialDeletion
from ._scram import UserScramCredentialsDescription as UserScramCredentialsDescription
from ._scram import UserScramCredentialUpsertion as UserScramCredentialUpsertion

class AdminClient(_AdminClientImpl):
    def __init__(self, conf: dict[str, Any]) -> None: ...
    def create_topics(
        self,
        new_topics: list["NewTopic"],
        operation_timeout: float = 0,
        request_timeout: float = 1000.0,
        validate_only: bool = False,
    ) -> dict[str, "Future[NewTopic | None]"]: ...
    def delete_topics(
        self, topics: list[str], operation_timeout: float = 0, request_timeout: float = 1000.0
    ) -> dict[str, "Future[NewTopic | None]"]: ...
    def list_topics(self, *args, **kwargs) -> list["TopicMetadata | None"]: ...
    def list_groups(self, *args, **kwargs) -> "ListConsumerGroupsResult | None": ...
    def create_partitions(
        self, new_partitions: list["NewPartitions"], operation_timeout: float = 0, request_timeout: float = 1000.0
    ) -> dict[str, "Future[NewTopic | None]"]: ...
    def describe_configs(
        self, resources: list["ConfigResource"], request_timeout: float = 1000.0
    ) -> dict["ConfigResource", "Future[dict[str, ConfigEntry]]"]: ...
    def alter_configs(
        self, resources: list["ConfigResource"], request_timeout: float = 1000.0, validate_only: bool = False
    ) -> dict["ConfigResource", "Future[None | KafkaException]"]: ...
    def incremental_alter_configs(
        self,
        resources: list["ConfigResource"],
        request_timeout: float = 1000.0,
        validate_only: bool = False,
        broker: int = 0,
    ) -> dict["ConfigResource", "Future[None | KafkaException]"]: ...
    def create_acls(
        self, acls: list["AclBinding"], request_timeout: float = 1000.0, **kwargs
    ) -> dict["AclBinding", "Future[None | KafkaException]"]: ...
    def describe_acls(
        self, acl_binding_filter: "AclBindingFilter", request_timeout: float = 1000.0
    ) -> "Future[list[AclBinding]]": ...
    def delete_acls(
        self, acl_binding_filters: list["AclBindingFilter"], request_timeout: float = 1000.0
    ) -> dict["AclBindingFilter", "Future[dict[AclBindingFilter, list[AclBinding]]]"]: ...
    def list_consumer_groups(
        self, request_timeout: float = 1000.0, states: set["ConsumerGroupState"] | None = None
    ) -> "Future[ListConsumerGroupsResult]": ...
    def describe_consumer_groups(
        self, group_ids: list[str], request_timeout: float = 1000.0
    ) -> dict[str, "Future[dict[str, ConsumerGroupDescription]]"]: ...
    def delete_consumer_groups(
        self, group_ids: list[str], request_timeout: float = 1000.0
    ) -> dict[str, "Future[None]"]: ...
    def list_consumer_group_offsets(
        self,
        list_consumer_group_offsets_request: list["ConsumerGroupTopicPartitions"],
        require_stable: bool = False,
        request_timeout: float = 1000.0,
    ) -> dict[str, "Future[ListConsumerGroupsResult]"]: ...
    def alter_consumer_group_offsets(
        self,
        alter_consumer_group_offsets_request: list["ConsumerGroupTopicPartitions"],
        request_timeout: float = 1000.0,
    ) -> dict[str, "Future[ConsumerGroupTopicPartitions]"]: ...
    def set_sasl_credentials(self, username: str, password: str) -> None: ...
    def describe_user_scram_credentials(
        self, users: list[str], request_timeout: float = 1000.0
    ) -> dict[str, "Future[UserScramCredentialsDescription | KafkaException]"]: ...
    def alter_user_scram_credentials(
        self, alterations: list["UserScramCredentialAlteration"], request_timeout: float = 1000.0
    ) -> dict[str, "Future[None | KafkaException]"]: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from ..serialization import SerializationContext
from .schema_registry_client import RegisteredSchema as RegisteredSchema
from .schema_registry_client import Schema as Schema
from .schema_registry_client import SchemaReference as SchemaReference
from .schema_registry_client import SchemaRegistryClient as SchemaRegistryClient
from .schema_registry_client import SchemaRegistryError as SchemaRegistryError

def topic_subject_name_strategy(ctx: SerializationContext, record_name: str): ...
def topic_record_subject_name_strategy(ctx: SerializationContext, record_name: str): ...
def record_subject_name_strategy(ctx: SerializationContext, record_name: str): ...
def reference_subject_name_strategy(ctx: SerializationContext, schema_ref: SchemaReference): ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import Enum
from typing import TYPE_CHECKING, ClassVar, Literal

if TYPE_CHECKING:
    from ..cimpl import TopicPartition

class Node:
    id_string: ClassVar[str]

    def __init__(self, id: int, host: str, port: int, rack: str | None = None) -> None: ...

class ConsumerGroupTopicPartitions:
    def __init__(self, group_id: str, topic_partitions: list["TopicPartition"] | None = None) -> None: ...

class ConsumerGroupState(Enum):
    UNKNOWN = Literal["CONSUMER_GROUP_STATE_UNKNOWN"]
    UNKOWN = Literal["UNKNOWN"]
    PREPARING_REBALANCING = Literal["CONSUMER_GROUP_STATE_PREPARING_REBALANCE"]
    COMPLETING_REBALANCING = Literal["CONSUMER_GROUP_STATE_COMPLETING_REBALANCE"]
    STABLE = Literal["CONSUMER_GROUP_STATE_STABLE"]
    DEAD = Literal["CONSUMER_GROUP_STATE_DEAD"]
    EMPTY = Literal["CONSUMER_GROUP_STATE_EMPTY"]
    def __lt__(self, other) -> bool: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import ClassVar

from ..error import KafkaException

class MessageField:
    NONE: str
    KEY: str
    VALUE: str

class SerializationContext:
    topic: ClassVar[str]
    field: ClassVar[str]
    headers: ClassVar[str]

    def __init__(self, topic: str, field: str, headers: list[tuple] | None = None) -> None: ...

class SerializationError(KafkaException): ...

class Serializer:
    def __call__(self, obj: object, ctx: SerializationContext | None = None) -> None: ...

class Deserializer:
    def __call__(self, value: bytes, ctx: SerializationContext | None = None) -> None: ...

class DoubleSerializer(Serializer):
    def __call__(self, obj: object, ctx: SerializationContext | None = None) -> None: ...

class DoubleDeserializer(Deserializer):
    def __call__(self, value: bytes, ctx: SerializationContext | None = None) -> None: ...

class IntegerSerializer(Serializer):
    def __call__(self, obj, ctx: SerializationContext | None = None) -> None: ...

class IntegerDeserializer(Deserializer):
    def __call__(self, value: bytes, ctx: SerializationContext | None = None) -> None: ...

class StringSerializer(Serializer):
    codec: ClassVar[str]

    def __init__(self, codec: str = ...) -> None: ...
    def __call__(self, obj: object, ctx: SerializationContext | None = None) -> None: ...

class StringDeserializer(Deserializer):
    codec: ClassVar[str]

    def __init__(self, codec: str = "utf_8") -> None: ...
    def __call__(self, value: bytes, ctx: SerializationContext | None = None) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import Any, ClassVar

from .._model import ConsumerGroupState as ConsumerGroupState
from ..error import KafkaError as KafkaError

class ClusterMetadata:
    cluster_id: ClassVar[str]
    controller_id: ClassVar[int] = -1
    brokers: ClassVar[dict[int, "BrokerMetadata"] | None] = None
    topics: ClassVar[dict[int, "TopicMetadata"] | None] = None
    orig_broker_id: ClassVar[int] = -1
    orig_broker_name: ClassVar[str] = ""
    def __init__(self) -> None: ...

class BrokerMetadata:
    id: ClassVar[int] = -1
    host: ClassVar[str | None] = None
    port: ClassVar[int] = -1
    def __init__(self) -> None: ...

class TopicMetadata:
    topic: ClassVar[str | None] = None
    partitions: ClassVar[dict[int, "PartitionMetadata"] | None] = None
    error: ClassVar["KafkaError" | None] = None
    def __init__(self) -> None: ...

class PartitionMetadata:
    id: ClassVar[int]
    leader: ClassVar[int]
    replicas: ClassVar[list[int] | None] = None
    isrs: ClassVar[list[int] | None] = None
    error: ClassVar["KafkaError" | None] = None
    def __init__(self) -> None: ...

class GroupMember:
    id: ClassVar[int | None] = None
    client_id: ClassVar[str | None] = None
    client_host: ClassVar[str | None] = None
    metadata: ClassVar[Any | None] = None
    assignment: ClassVar[Any | None] = None
    def __init__(self) -> None: ...

class GroupMetadata:
    broker: ClassVar["BrokerMetadata" | None] = None
    id: ClassVar[str | None] = None
    error: ClassVar["KafkaError" | None] = None
    state: ClassVar["ConsumerGroupState" | None] = None
    protocol_type: ClassVar[Any | None] = None
    protocol: ClassVar[Any] = None
    members: ClassVar[list | None] = None
    def __init__(self) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import Enum
from typing import Literal, Type

from ..cimpl import KafkaError as KafkaError
from ._resource import ResourceType as ResourceType

class AlterConfigOpType(Enum):
    SET: Literal["ALTER_CONFIG_OP_TYPE_SET"]
    DELETE: Literal["ALTER_CONFIG_OP_TYPE_DELETE"]
    APPEND: Literal["ALTER_CONFIG_OP_TYPE_APPEND"]
    SUBTRACT: Literal["ALTER_CONFIG_OP_TYPE_SUBTRACT"]

class ConfigSource(Enum):
    UNKNOWN_CONFIG: Literal["CONFIG_SOURCE_UNKNOWN_CONFIG"]
    DYNAMIC_TOPIC_CONFIG: Literal["CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG"]
    DYNAMIC_BROKER_CONFIG: Literal["CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG"]
    DYNAMIC_DEFAULT_BROKER_CONFIG: Literal["CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG"]
    STATIC_BROKER_CONFIG: Literal["CONFIG_SOURCE_STATIC_BROKER_CONFIG"]
    DEFAULT_CONFIG: Literal["CONFIG_SOURCE_DEFAULT_CONFIG"]

class ConfigEntry:
    def __init__(
        self,
        name: str,
        value,
        source=Literal["UNKNOWN_CONFIG"],
        is_read_only: bool = False,
        is_default: bool = False,
        is_sensitive: bool = False,
        is_synonym: bool = False,
        synonyms: list["ConfigEntry"] | None = None,
        incremental_operation: "AlterConfigOpType" | None = None,
    ) -> None: ...

class ConfigResource:
    def __init__(
        self,
        restype: Type["ResourceType"],
        name: str,
        set_config: dict[str, str] | None = None,
        described_configs: dict | None = ...,
        error: "KafkaError" | None = None,
        incremental_configs: list["ConfigEntry"] | None = None,
    ) -> None: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: "ConfigResource") -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __len__(self) -> int: ...
    def set_config(self, name: str, value, overwrite: bool = True) -> None: ...
    def add_incremental_config(self, config_entry: "ConfigEntry") -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from types import TracebackType
from typing import Any, Callable, ClassVar

def libversion() -> tuple[str, int]: ...
def version() -> tuple[str, int]: ...

# Constants
ACL_OPERATION_ALL: int
ACL_OPERATION_ALTER: int
ACL_OPERATION_ALTER_CONFIGS: int
ACL_OPERATION_ANY: int
ACL_OPERATION_CLUSTER_ACTION: int
ACL_OPERATION_CREATE: int
ACL_OPERATION_DELETE: int
ACL_OPERATION_DESCRIBE: int
ACL_OPERATION_DESCRIBE_CONFIGS: int
ACL_OPERATION_IDEMPOTENT_WRITE: int
ACL_OPERATION_READ: int
ACL_OPERATION_UNKNOWN: int
ACL_OPERATION_WRITE: int
ACL_PERMISSION_TYPE_ALLOW: int
ACL_PERMISSION_TYPE_ANY: int
ACL_PERMISSION_TYPE_DENY: int
ACL_PERMISSION_TYPE_UNKNOWN: int
CONFIG_SOURCE_DEFAULT_CONFIG: int
CONFIG_SOURCE_DYNAMIC_BROKER_CONFIG: int
CONFIG_SOURCE_DYNAMIC_DEFAULT_BROKER_CONFIG: int
CONFIG_SOURCE_DYNAMIC_TOPIC_CONFIG: int
CONFIG_SOURCE_STATIC_BROKER_CONFIG: int
CONFIG_SOURCE_UNKNOWN_CONFIG: int
CONSUMER_GROUP_STATE_COMPLETING_REBALANCE: int
CONSUMER_GROUP_STATE_DEAD: int
CONSUMER_GROUP_STATE_EMPTY: int
CONSUMER_GROUP_STATE_PREPARING_REBALANCE: int
CONSUMER_GROUP_STATE_STABLE: int
CONSUMER_GROUP_STATE_UNKNOWN: int
OFFSET_BEGINNING: int
OFFSET_END: int
OFFSET_INVALID: int
OFFSET_STORED: int
RESOURCE_ANY: int
RESOURCE_BROKER: int
RESOURCE_GROUP: int
RESOURCE_PATTERN_ANY: int
RESOURCE_PATTERN_LITERAL: int
RESOURCE_PATTERN_MATCH: int
RESOURCE_PATTERN_PREFIXED: int
RESOURCE_PATTERN_UNKNOWN: int
RESOURCE_TOPIC: int
RESOURCE_UNKNOWN: int
TIMESTAMP_CREATE_TIME: int
TIMESTAMP_LOG_APPEND_TIME: int
TIMESTAMP_NOT_AVAILABLE: int

class KafkaError:
    # Class Private Vars
    _ALL_BROKERS_DOWN: ClassVar[int]
    _APPLICATION: ClassVar[int]
    _ASSIGNMENT_LOST: ClassVar[int]
    _ASSIGN_PARTITIONS: ClassVar[int]
    _AUTHENTICATION: ClassVar[int]
    _AUTO_OFFSET_RESET: ClassVar[int]
    _BAD_COMPRESSION: ClassVar[int]
    _BAD_MSG: ClassVar[int]
    _CONFLICT: ClassVar[int]
    _CRIT_SYS_RESOURCE: ClassVar[int]
    _DESTROY: ClassVar[int]
    _EXISTING_SUBSCRIPTION: ClassVar[int]
    _FAIL: ClassVar[int]
    _FATAL: ClassVar[int]
    _FENCED: ClassVar[int]
    _FS: ClassVar[int]
    _GAPLESS_GUARANTEE: ClassVar[int]
    _INCONSISTENT: ClassVar[int]
    _INTR: ClassVar[int]
    _INVALID_ARG: ClassVar[int]
    _INVALID_TYPE: ClassVar[int]
    _IN_PROGRESS: ClassVar[int]
    _ISR_INSUFF: ClassVar[int]
    _KEY_DESERIALIZATION: ClassVar[int]
    _KEY_SERIALIZATION: ClassVar[int]
    _LOG_TRUNCATION: ClassVar[int]
    _MAX_POLL_EXCEEDED: ClassVar[int]
    _MSG_TIMED_OUT: ClassVar[int]
    _NODE_UPDATE: ClassVar[int]
    _NOENT: ClassVar[int]
    _NOOP: ClassVar[int]
    _NOT_CONFIGURED: ClassVar[int]
    _NOT_IMPLEMENTED: ClassVar[int]
    _NO_OFFSET: ClassVar[int]
    _OUTDATED: ClassVar[int]
    _PARTIAL: ClassVar[int]
    _PARTITION_EOF: ClassVar[int]
    _PREV_IN_PROGRESS: ClassVar[int]
    _PURGE_INFLIGHT: ClassVar[int]
    _PURGE_QUEUE: ClassVar[int]
    _QUEUE_FULL: ClassVar[int]
    _READ_ONLY: ClassVar[int]
    _RESOLVE: ClassVar[int]
    _RETRY: ClassVar[int]
    _REVOKE_PARTITIONS: ClassVar[int]
    _SSL: ClassVar[int]
    _STATE: ClassVar[int]
    _TIMED_OUT: ClassVar[int]
    _TIMED_OUT_QUEUE: ClassVar[int]
    _TRANSPORT: ClassVar[int]
    _UNDERFLOW: ClassVar[int]
    _UNKNOWN_BROKER: ClassVar[int]
    _UNKNOWN_GROUP: ClassVar[int]
    _UNKNOWN_PARTITION: ClassVar[int]
    _UNKNOWN_PROTOCOL: ClassVar[int]
    _UNKNOWN_TOPIC: ClassVar[int]
    _UNSUPPORTED_FEATURE: ClassVar[int]
    _VALUE_DESERIALIZATION: ClassVar[int]
    _VALUE_SERIALIZATION: ClassVar[int]
    _WAIT_CACHE: ClassVar[int]
    _WAIT_COORD: ClassVar[int]

    # Class Vars
    BROKER_NOT_AVAILABLE: ClassVar[int]
    CLUSTER_AUTHORIZATION_FAILED: ClassVar[int]
    CONCURRENT_TRANSACTIONS: ClassVar[int]
    COORDINATOR_LOAD_IN_PROGRESS: ClassVar[int]
    COORDINATOR_NOT_AVAILABLE: ClassVar[int]
    DELEGATION_TOKEN_AUTH_DISABLED: ClassVar[int]
    DELEGATION_TOKEN_AUTHORIZATION_FAILED: ClassVar[int]
    DELEGATION_TOKEN_EXPIRED: ClassVar[int]
    DELEGATION_TOKEN_NOT_FOUND: ClassVar[int]
    DELEGATION_TOKEN_OWNER_MISMATCH: ClassVar[int]
    DELEGATION_TOKEN_REQUEST_NOT_ALLOWED: ClassVar[int]
    DUPLICATE_RESOURCE: ClassVar[int]
    DUPLICATE_SEQUENCE_NUMBER: ClassVar[int]
    ELECTION_NOT_NEEDED: ClassVar[int]
    ELIGIBLE_LEADERS_NOT_AVAILABLE: ClassVar[int]
    FEATURE_UPDATE_FAILED: ClassVar[int]
    FENCED_INSTANCE_ID: ClassVar[int]
    FENCED_LEADER_EPOCH: ClassVar[int]
    FETCH_SESSION_ID_NOT_FOUND: ClassVar[int]
    GROUP_AUTHORIZATION_FAILED: ClassVar[int]
    GROUP_ID_NOT_FOUND: ClassVar[int]
    GROUP_MAX_SIZE_REACHED: ClassVar[int]
    GROUP_SUBSCRIBED_TO_TOPIC: ClassVar[int]
    ILLEGAL_GENERATION: ClassVar[int]
    ILLEGAL_SASL_STATE: ClassVar[int]
    INCONSISTENT_GROUP_PROTOCOL: ClassVar[int]
    INCONSISTENT_VOTER_SET: ClassVar[int]
    INVALID_COMMIT_OFFSET_SIZE: ClassVar[int]
    INVALID_CONFIG: ClassVar[int]
    INVALID_FETCH_SESSION_EPOCH: ClassVar[int]
    INVALID_GROUP_ID: ClassVar[int]
    INVALID_MSG: ClassVar[int]
    INVALID_MSG_SIZE: ClassVar[int]
    INVALID_PARTITIONS: ClassVar[int]
    INVALID_PRINCIPAL_TYPE: ClassVar[int]
    INVALID_PRODUCER_EPOCH: ClassVar[int]
    INVALID_PRODUCER_ID_MAPPING: ClassVar[int]
    INVALID_RECORD: ClassVar[int]
    INVALID_REPLICA_ASSIGNMENT: ClassVar[int]
    INVALID_REPLICATION_FACTOR: ClassVar[int]
    INVALID_REQUEST: ClassVar[int]
    INVALID_REQUIRED_ACKS: ClassVar[int]
    INVALID_SESSION_TIMEOUT: ClassVar[int]
    INVALID_TIMESTAMP: ClassVar[int]
    INVALID_TRANSACTION_TIMEOUT: ClassVar[int]
    INVALID_TXN_STATE: ClassVar[int]
    INVALID_UPDATE_VERSION: ClassVar[int]
    KAFKA_STORAGE_ERROR: ClassVar[int]
    LEADER_NOT_AVAILABLE: ClassVar[int]
    LISTENER_NOT_FOUND: ClassVar[int]
    LOG_DIR_NOT_FOUND: ClassVar[int]
    MEMBER_ID_REQUIRED: ClassVar[int]
    MSG_SIZE_TOO_LARGE: ClassVar[int]
    NETWORK_EXCEPTION: ClassVar[int]
    NO_ERROR: ClassVar[int]
    NON_EMPTY_GROUP: ClassVar[int]
    NO_REASSIGNMENT_IN_PROGRESS: ClassVar[int]
    NOT_CONTROLLER: ClassVar[int]
    NOT_COORDINATOR: ClassVar[int]
    NOT_ENOUGH_REPLICAS: ClassVar[int]
    NOT_ENOUGH_REPLICAS_AFTER_APPEND: ClassVar[int]
    NOT_LEADER_FOR_PARTITION: ClassVar[int]
    OFFSET_METADATA_TOO_LARGE: ClassVar[int]
    OFFSET_NOT_AVAILABLE: ClassVar[int]
    OFFSET_OUT_OF_RANGE: ClassVar[int]
    OPERATION_NOT_ATTEMPTED: ClassVar[int]
    OUT_OF_ORDER_SEQUENCE_NUMBER: ClassVar[int]
    POLICY_VIOLATION: ClassVar[int]
    PREFERRED_LEADER_NOT_AVAILABLE: ClassVar[int]
    PRINCIPAL_DESERIALIZATION_FAILURE: ClassVar[int]
    PRODUCER_FENCED: ClassVar[int]
    REASSIGNMENT_IN_PROGRESS: ClassVar[int]
    REBALANCE_IN_PROGRESS: ClassVar[int]
    RECORD_LIST_TOO_LARGE: ClassVar[int]
    REPLICA_NOT_AVAILABLE: ClassVar[int]
    REQUEST_TIMED_OUT: ClassVar[int]
    RESOURCE_NOT_FOUND: ClassVar[int]
    SASL_AUTHENTICATION_FAILED: ClassVar[int]
    SECURITY_DISABLED: ClassVar[int]
    STALE_BROKER_EPOCH: ClassVar[int]
    STALE_CTRL_EPOCH: ClassVar[int]
    THROTTLING_QUOTA_EXCEEDED: ClassVar[int]
    TOPIC_ALREADY_EXISTS: ClassVar[int]
    TOPIC_AUTHORIZATION_FAILED: ClassVar[int]
    TOPIC_DELETION_DISABLED: ClassVar[int]
    TOPIC_EXCEPTION: ClassVar[int]
    TRANSACTIONAL_ID_AUTHORIZATION_FAILED: ClassVar[int]
    TRANSACTION_COORDINATOR_FENCED: ClassVar[int]
    UNACCEPTABLE_CREDENTIAL: ClassVar[int]
    UNKNOWN: ClassVar[int]
    UNKNOWN_LEADER_EPOCH: ClassVar[int]
    UNKNOWN_MEMBER_ID: ClassVar[int]
    UNKNOWN_PRODUCER_ID: ClassVar[int]
    UNKNOWN_TOPIC_OR_PART: ClassVar[int]
    UNSTABLE_OFFSET_COMMIT: ClassVar[int]
    UNSUPPORTED_COMPRESSION_TYPE: ClassVar[int]
    UNSUPPORTED_FOR_MESSAGE_FORMAT: ClassVar[int]
    UNSUPPORTED_SASL_MECHANISM: ClassVar[int]
    UNSUPPORTED_VERSION: ClassVar[int]

    def __init__(
        self,
        error_code: int,
        reason: str | None = None,
        fatal: bool = False,
        retriable: bool = False,
        txn_requires_abort: bool = False,
    ) -> None:
        """
        Note:
            - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#kafkaerror
        """
        ...
    def code(self) -> int: ...
    def fatal(self) -> bool: ...
    def name(self) -> str: ...
    def retriable(self) -> bool: ...
    def str(self) -> str: ...
    def txn_requires_abort(self) -> bool: ...
    def __eq__(self, other: "object") -> bool: ...
    def __ge__(self, other: "KafkaError") -> bool: ...
    def __gt__(self, other: "KafkaError") -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other: "KafkaError") -> bool: ...
    def __lt__(self, other: "KafkaError") -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class KafkaException(Exception):
    # Attributes
    args: tuple[str | KafkaError]

    # Methods
    def __init__(self, error: KafkaError | None = None, *args, **kwargs) -> None:
        """
        Note:
            - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#kafkaexception
            - Kafka exception that wraps the KafkaError class. Use `exception.args[0]` to extract the `KafkaError` object
        """
        ...
    def add_note(self, note: str) -> None: ...
    def with_traceback(self, traceback: TracebackType | None) -> Any: ...

class _AdminClientImpl:
    """
    Note:
        - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#module-confluent_kafka.admin
    """

    def __init__(self, *args, **kwargs) -> None: ...
    def alter_configs(self, *args, **kwargs) -> Any: ...
    def alter_consumer_group_offsets(self, *args, **kwargs) -> Any: ...
    def alter_user_scram_credentials(self, *args, **kwargs) -> Any: ...
    def create_acls(self, *args, **kwargs) -> Any: ...
    def create_partitions(self, *args, **kwargs) -> Any: ...
    def create_topics(self, *args, **kwargs) -> Any: ...
    def delete_acls(self, *args, **kwargs) -> Any: ...
    def delete_consumer_groups(self, *args, **kwargs) -> Any: ...
    def delete_topics(self, *args, **kwargs) -> Any: ...
    def describe_acls(self, *args, **kwargs) -> Any: ...
    def describe_configs(self, *args, **kwargs) -> Any: ...
    def describe_consumer_groups(self, *args, **kwargs) -> Any: ...
    def describe_user_scram_credentials(self, *args, **kwargs) -> Any: ...
    def incremental_alter_configs(self, *args, **kwargs) -> Any: ...
    def list_consumer_group_offsets(self, *args, **kwargs) -> Any: ...
    def list_consumer_groups(self, *args, **kwargs) -> Any: ...
    def list_groups(self, *args, **kwargs) -> Any: ...
    def list_topics(self, *args, **kwargs) -> Any: ...
    def poll(self, *args, **kwargs) -> Any: ...
    def set_sasl_credentials(self, username, password) -> Any: ...
    def __len__(self) -> int: ...

class Message:
    """
    Note:
        doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#message
    """

    # Methods
    def error(self) -> KafkaError | None: ...
    def headers(self) -> list[tuple[str, str | bytes]] | None: ...
    def key(self) -> str | bytes | None: ...
    def latency(self) -> float | None: ...
    def leader_epoch(self) -> int | None: ...
    def offset(self) -> int | None: ...
    def partition(self) -> int | None: ...
    def set_headers(self, value: list[tuple[str, str | bytes]]) -> None: ...
    def set_key(self, value: str | bytes) -> None: ...
    def set_value(self, value: str | bytes) -> None: ...
    def timestamp(self) -> tuple[int, int]: ...
    def topic(self) -> str | None: ...
    def value(self) -> str | bytes | None: ...
    def __len__(self) -> int: ...

class NewTopic:
    def __init__(
        self,
        topic: str,
        num_partitions: int = -1,
        replication_factor: int = -1,
        replica_assignment: list[list[str]] | None = None,
        config: dict[str, str] | None = None,
    ) -> None:
        """
        Note:
            - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#newtopic
            - config reference: http://kafka.apache.org/documentation.html#topicconfigs
        """
        def __eq__(self, other: "NewTopic") -> bool: ...
        def __ge__(self, other: "NewTopic") -> bool: ...
        def __gt__(self, other: "NewTopic") -> bool: ...
        def __hash__(self) -> int: ...
        def __le__(self, other: "NewTopic") -> bool: ...
        def __lt__(self, other: "NewTopic") -> bool: ...
        def __ne__(self, other: "NewTopic") -> bool: ...

class NewPartitions:
    def __init__(self, topic: str, new_total_count: int, replica_assignment: list[list[str]] | None = None) -> None:
        """
        Note:
            - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#newpartitions
        """
        def __eq__(self, other: "NewPartitions") -> bool: ...
        def __ge__(self, other: "NewPartitions") -> bool: ...
        def __gt__(self, other: "NewPartitions") -> bool: ...
        def __hash__(self) -> int: ...
        def __le__(self, other: "NewPartitions") -> bool: ...
        def __lt__(self, other: "NewPartitions") -> bool: ...
        def __ne__(self, other: "NewPartitions") -> bool: ...

class TopicPartition:
    def __init__(
        self,
        topic: str,
        partition: int = -1,
        offset: int = -1001,
        metadata: str | None = None,
        leader_epoch: int | None = None,
    ) -> None:
        """
        Note:
            - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#topicpartition
        """
        def __eq__(self, other: "TopicPartition") -> bool: ...
        def __ge__(self, other: "TopicPartition") -> bool: ...
        def __gt__(self, other: "TopicPartition") -> bool: ...
        def __hash__(self) -> int: ...
        def __le__(self, other: "TopicPartition") -> bool: ...
        def __lt__(self, other: "TopicPartition") -> bool: ...
        def __ne__(self, other: "TopicPartition") -> bool: ...

class Producer:
    """
    Note:
        - doc: https://docs.confluent.io/platform/current/clients/confluent-kafka-python/html/index.html#producer
        - Asynchronous Kafka Producer
    """

    # Methods
    def __init__(self, config: dict[str, Any]) -> None: ...
    def abort_transaction(self, timeout: float | None = None) -> None: ...
    def begin_transaction(self) -> None: ...
    def commit_transaction(self, timeout: float | None = None) -> None:
        """
        Note:
            - timeout unit - second
        """
        ...
    def flush(self, timeout: float | None = None) -> int:
        """
        Note:
            - timeout unit - second
        """
        ...
    def init_transactions(self, timeout: float | None = None) -> None:
        """
        Note:
            - timeout unit - second
        """
        ...
    def list_topics(self, topic: str | None = None, timeout: float = -1) -> Any: ...
    def poll(self, timeout: float | None = None) -> int:
        """
        Note:
            - timeout unit - second
        """
        ...
    def produce(
        self,
        topic: str,
        value: str | bytes | None = None,
        key: str | bytes | None = None,
        partition: int | None = None,
        on_delivery: Callable | None = None,
        timestamp: int = 0,
        headers: dict[str, str | bytes] | list[tuple[str, str | bytes]] | None = None,
    ) -> None: ...
    def purge(self, in_queue: bool = True, in_flight: bool = True, blocking: bool = True) -> None: ...
    def send_offsets_to_transactions(
        self, positions: list[TopicPartition], group_metadata: object, timeout: float | None = None
    ) -> None:
        """
        Note:
            - timeout unit - second
        """
        ...
    def set_sasl_credentials(self, username: str, password: str) -> None: ...
    def __len__(self) -> int: ...

class Consumer:
    def __init__(self, config: dict[str, Any]) -> None: ...
    def assign(self, partitions: list[TopicPartition]) -> None: ...
    def assignment(self) -> list[TopicPartition]: ...
    def close(self) -> None: ...
    def commit(
        self,
        message: Message | None = None,
        offsets: list[TopicPartition] | None = None,
        asynchronous: bool = True,
    ) -> list[TopicPartition] | None: ...
    def committed(
        self, partitions: list[TopicPartition], timeout: float | int | None = None
    ) -> list[TopicPartition]: ...
    def consume(self, num_messages: int = 1, timeout: float | int = -1) -> list[TopicPartition]: ...
    def consumer_group_metadata(self) -> object: ...
    def get_watermark_offsets(
        self, partition: TopicPartition, timeout: float | int | None = None, cached: bool = False
    ) -> tuple[int, int]: ...
    def incremental_assign(self, partitions: list[TopicPartition]) -> None: ...
    def incremental_unassign(self, partitions: list[TopicPartition]) -> None: ...
    def list_topics(self, topics: str | None = None, timeout: float | int = -1) -> Any: ...
    def memberid(self) -> str | None: ...
    def offsets_for_times(
        self, partitions: list[TopicPartition], timeout: float | int | None = None
    ) -> list[TopicPartition]: ...
    def pause(self, partitions: list[TopicPartition]) -> None: ...
    def poll(self, timeout: float | int | None) -> Message | None: ...
    def position(self, partitions: list[TopicPartition]) -> list[TopicPartition]: ...
    def resume(self, partitions: list[TopicPartition]) -> None: ...
    def seek(self, partition: TopicPartition) -> None: ...
    def set_sasl_credentials(self, username: str, password: str) -> None: ...
    def store_offsets(self, message: Message | None = None, offsets: list[TopicPartition] | None = None) -> None: ...
    def subscribe(
        self,
        topics: list[str],
        on_assign: Callable | None = None,
        on_revoke: Callable | None = None,
        on_lost: Callable | None = None,
    ) -> None: ...
    def unassign(self) -> None: ...
    def unsubscribe(self) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import ClassVar

from ..cimpl import Message

class ClientError(Exception):
    """Error thrown by Schema Registry clients"""

    message: ClassVar[Message]
    http_code: ClassVar[int]

    def __init__(self, message, http_code: int | None = None) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from logging import Logger
from threading import Lock
from typing import ClassVar, DefaultDict, Literal

# pypi/conda library
from requests import Session

from .error import SchemaRegistryError as SchemaRegistryError

log: Logger
VALID_AUTH_PROVIDERS: Literal["URL", "USER_INFO"]

class _RestClient:
    session: Session
    base_url: ClassVar[str]

    def __init__(self, conf: dict) -> None: ...
    def get(self, url: str, query: dict | None = ...): ...
    def post(self, url: str, body, **kwargs): ...
    def delete(self, url: str): ...
    def put(self, url: str, body: str | None = None): ...
    def send_request(
        self,
        url: str,
        method: Literal["GET", "POST", "DELETE", "PUT"],
        body: str | None = None,
        query: dict | None = None,
    ): ...

class _SchemaCache:
    lock: ClassVar[Lock]
    schema_id_index: ClassVar[dict]
    schema_index: ClassVar[dict]
    subject_schemas: DefaultDict[str, set]

    def __init__(self) -> None: ...
    def set(self, schema_id: int, schema: Schema, subject_name: str | None = None) -> None: ...
    def get_schema(self, schema_id: int) -> Schema | None: ...
    def get_schema_id_by_subject(self, subject, schema: Schema) -> Schema | None: ...

class SchemaRegistryClient:
    def __init__(self, conf: dict) -> None: ...
    def __enter__(self) -> SchemaRegistryClient: ...
    def __exit__(self, *args) -> None: ...
    def register_schema(self, subject_name: str, schema: Schema, normalize_schemas: bool = False) -> int: ...
    def get_schema(self, schema_id) -> Schema: ...
    def lookup_schema(self, subject_name: str, schema: Schema, normalize_schemas: bool = False) -> RegisteredSchema: ...
    def get_subjects(self) -> list[str]: ...
    def delete_subject(self, subject_name: str, permanent: bool = ...) -> list[int]: ...
    def get_latest_version(self, subject_name: str) -> RegisteredSchema: ...
    def get_version(self, subject_name: str, version: int) -> RegisteredSchema: ...
    def get_versions(self, subject_name: str) -> list[int]: ...
    def delete_version(self, subject_name: str, version: int) -> int: ...
    def set_compatibility(self, subject_name: str | None = None, level: str | None = None) -> str: ...
    def get_compatibility(self, subject_name: str | None = None) -> str: ...
    def test_compatibility(
        self,
        subject_name: str,
        schema: Schema,
        version: int | Literal["latest"] = "latest",
    ) -> bool: ...

class Schema:
    schema_str: ClassVar[str]
    schema_type: ClassVar[str]
    references: ClassVar[list[SchemaReference]]

    def __init__(self, schema_str: str, schema_type: str, references: list[SchemaReference] | None = None) -> None: ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...

class RegisteredSchema:
    schema_id: ClassVar[int]
    schema: ClassVar[Schema]
    subject: ClassVar[str]
    version: ClassVar[int]

    def __init__(self, schema_id: int, schema: Schema, subject: str, version: int) -> None: ...

class SchemaReference:
    name: ClassVar[str]
    subject: ClassVar[str]
    version: ClassVar[str]

    def __init__(self, name: str, subject: str, version: str) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from io import BytesIO
from logging import Logger
from typing import TYPE_CHECKING, Any, Callable

from ...schema_registry.schema_registry_client import SchemaRegistryClient
from .. import ClientError as ClientError
from . import KeySerializerError as KeySerializerError
from . import SerializerError as SerializerError
from . import ValueSerializerError as ValueSerializerError

log: Logger
MAGIC_BYTE: int
HAS_FAST: bool

if TYPE_CHECKING:
    """Helping users who installed avro package can get type hints"""
    try:
        # pypi/conda library
        from avro.schema import Schema  # type: ignore # this cannot be resolved if avro isn't installed
    except ImportError:
        # standard library
        from typing import Any as Schema  # type: ignore

class ContextStringIO(BytesIO):
    def __enter__(self): ...
    def __exit__(self, *args): ...

class MessageSerializer:
    registry_client: SchemaRegistryClient
    id_to_decoder_func: dict[int, Callable]
    id_to_writers: dict[int, bytes]
    reader_key_schema: str | bytes | None = None
    reader_value_schema: str | bytes | None = None

    def __init__(
        self,
        registry_client,
        reader_key_schema: Any | None = None,
        reader_value_schema: Any | None = None,
    ) -> None: ...
    def encode_record_with_schema(self, topic: str, schema: "Schema", record: dict, is_key: bool = ...) -> bytes: ...
    def encode_record_with_schema_id(self, schema_id: int, record: dict, is_key: bool = ...) -> Callable: ...
    def decode_message(self, message: str | bytes | None = None, is_key: bool = ...) -> dict[str, Any]: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from logging import Logger
from pathlib import Path
from typing import TYPE_CHECKING, DefaultDict, Literal

from .error import ClientError as ClientError
from .load import loads as loads

if TYPE_CHECKING:
    """Helping users who installed avro package can get type hints"""
    try:
        # pypi/conda library
        from avro.schema import Schema  # type: ignore # this cannot be resolved if avro isn't installed
    except ImportError:
        # standard library
        from typing import Any as Schema  # type: ignore

VALID_LEVELS = Literal["NONE"] | Literal["FULL"] | Literal["FORWARD"] | Literal["BACKWARD"]
VALID_METHODS = Literal["GET"] | Literal["POST"] | Literal["PUT"] | Literal["DELETE"]
VALID_AUTH_PROVIDERS = Literal["URL"] | Literal["USER_INFO"] | Literal["SASL_INHERIT"]
ACCEPT_HDR: str
log: Logger

class CachedSchemaRegistryClient:
    url: str | dict
    subject_to_schema_ids: DefaultDict[str, dict]
    id_to_schema: DefaultDict[str, dict]
    subject_to_schema_versions: DefaultDict[str, dict]
    auto_register_schemas: bool

    def __init__(
        self,
        url: str | dict,
        max_schemas_per_subject: int,
        ca_location: str | Path | None = None,
        cert_location: str | Path | None = None,
        key_location: str | Path | None = None,
    ) -> None: ...
    def __del__(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...
    def close(self) -> None: ...
    def register(self, subject: str, avro_schema: "Schema") -> int: ...
    def check_registration(self, subject: str, avro_schema: "Schema") -> int: ...
    def delete_subject(self, subject: str) -> int: ...
    def get_by_id(self, schema_id: int) -> "Schema": ...
    def get_latest_schema(self, subject: str) -> tuple[str, "Schema", int]: ...
    def get_by_version(self, subject: str, version) -> tuple[str, "Schema", int]: ...
    def get_version(self, subject: str, avro_schema: "Schema") -> int: ...
    def test_compatibility(self, subject: str, avro_schema: "Schema", version: str = "latest") -> bool: ...
    def update_compatibility(self, level: VALID_LEVELS, subject: str | None = None) -> None: ...
    def get_compatibility(self, subject: str | None = None) -> VALID_LEVELS: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from ..cimpl import KafkaError as KafkaError

class ValidationUtil:
    @staticmethod
    def check_multiple_not_none(obj, vars_to_check) -> None: ...
    @staticmethod
    def check_not_none(obj, param) -> None: ...
    @staticmethod
    def check_multiple_is_string(obj, vars_to_check) -> None: ...
    @staticmethod
    def check_is_string(obj, param) -> None: ...
    @staticmethod
    def check_kafka_errors(errors) -> None: ...
    @staticmethod
    def check_kafka_error(error) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from enum import Enum, EnumMeta
from typing import Any, Literal

from .._util import ConversionUtil as ConversionUtil
from .._util import ValidationUtil as ValidationUtil
from ._resource import ResourcePatternType as ResourcePatternType
from ._resource import ResourceType as ResourceType

class AclOperation(Enum):
    UNKNOWN: Literal["ACL_OPERATION_UNKNOWN"]
    ANY: Literal["ACL_OPERATION_ANY"]
    ALL: Literal["ACL_OPERATION_ALL"]
    READ: Literal["ACL_OPERATION_READ"]
    WRITE: Literal["ACL_OPERATION_WRITE"]
    CREATE: Literal["ACL_OPERATION_CREATE"]
    DELETE: Literal["ACL_OPERATION_DELETE"]
    ALTER: Literal["ACL_OPERATION_ALTER"]
    DESCRIBE: Literal["ACL_OPERATION_DESCRIBE"]
    CLUSTER_ACTION: Literal["ACL_OPERATION_CLUSTER_ACTION"]
    DESCRIBE_CONFIGS: Literal["ACL_OPERATION_DESCRIBE_CONFIGS"]
    ALTER_CONFIGS: Literal["ACL_OPERATION_ALTER_CONFIGS"]
    IDEMPOTENT_WRITE: Literal["ACL_OPERATION_IDEMPOTENT_WRITE"]
    def __lt__(self, other: "AclOperation") -> bool: ...

class AclPermissionType(Enum):
    UNKNOWN: Literal["ACL_PERMISSION_TYPE_UNKNOWN"]
    ANY: Literal["ACL_PERMISSION_TYPE_ANY"]
    DENY: Literal["ACL_PERMISSION_TYPE_DENY"]
    ALLOW: Literal["ACL_PERMISSION_TYPE_ALLOW"]
    def __lt__(self, other: "AclPermissionType") -> bool: ...

class AclBinding:
    def __init__(
        self,
        restype: "ResourceType",
        name: str,
        resource_pattern_type: "ResourcePatternType",
        principal: str,
        host: str,
        operation: "AclOperation",
        permission_type: "AclPermissionType",
    ) -> None: ...
    def _convert_enums(self) -> None: ...
    def _check_forbidden_enums(self, forbidden_enums: "EnumMeta") -> None: ...
    def _not_none_args(
        self,
    ) -> Literal["restype", "name", "resource_pattern_type", "principal", "host", "operation", "permission_type"]: ...
    def _string_args(self) -> Literal["name", "principal", "host"]: ...
    def _forbidden_enums(
        self,
    ) -> dict[
        str,
        Literal[
            "RESOURCE_ANY",
            "RESOURCE_PATTERN_ANY",
            "RESOURCE_PATTERN_MATCH",
            "ACL_OPERATION_ANY",
            "ACL_PERMISSION_TYPE_ANY",
        ],
    ]: ...
    def _convert_args(self) -> None: ...
    def _to_tuple(self) -> tuple[Any]: ...
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: "AclBinding") -> bool: ...
    def __eq__(self, other: object) -> bool: ...

class AclBindingFilter(AclBinding):
    def __init__(
        self,
        restype: "ResourceType",
        name: str,
        resource_pattern_type: "ResourcePatternType",
        principal: str,
        host: str,
        operation: "AclOperation",
        permission_type: "AclPermissionType",
    ) -> None: ...
    def _not_none_args(self) -> Literal["restype", "resource_pattern_type", "operation", "permission_type"]: ...
    def _forbidden_enums(  # type: ignore[override]
        self,
    ) -> dict[
        str,
        Literal["RESOURCE_UNKNOWN", "RESOURCE_PATTERN_UNKNOWN", "ACL_OPERATION_UNKNOWN", "ACL_PERMISSION_TYPE_UNKNOWN"],
    ]: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import Callable

from .cimpl import KafkaError, Message
from .cimpl import Producer as _ProducerImpl
from .error import KeySerializationError as KeySerializationError
from .error import ValueSerializationError as ValueSerializationError
from .serialization import MessageField as MessageField
from .serialization import SerializationContext as SerializationContext

class SerializingProducer(_ProducerImpl):
    def __init__(self, conf) -> None: ...
    def produce(
        self,
        topic: str,
        key: object | None = None,
        value: object | None = None,
        partition: int = -1,  # type: ignore[override]
        on_delivery: Callable[[KafkaError], Message] | None = None,
        timestamp: float = 0,
        headers: dict[str, bytes | None] | None = None,  # type: ignore[override]
    ) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

from .._model import ConsumerGroupState as ConsumerGroupState
from .._model import Node as Node
from .._util import ConversionUtil as ConversionUtil
from ..cimpl import TopicPartition as TopicPartition
from ..error import KafkaException as KafkaException

class ConsumerGroupListing:
    def __init__(
        self, group_id: str, is_simple_consumer_group: bool, state: ConsumerGroupState | None = None
    ) -> None: ...

class ListConsumerGroupsResult:
    def __init__(
        self, valid: list["ConsumerGroupListing"] | None = None, errors: list["KafkaException"] | None = None
    ) -> None: ...

class MemberAssignment:
    def __init__(self, topic_partitions: list["TopicPartition"] | None = None) -> None: ...

class MemberDescription:
    def __init__(
        self,
        member_id: str,
        client_id: str,
        host: str,
        assignment: "MemberAssignment",
        group_instance_id: str | None = None,
    ) -> None: ...

class ConsumerGroupDescription:
    def __init__(
        self,
        group_id: str,
        is_simple_consumer_group: bool,
        members: list["MemberDescription"],
        partition_assignor: str,
        state: "ConsumerGroupState",
        coordinator: "Node",
    ) -> None: ...



"""
types-confluent-kafka: A package providing type hints for the confluent-kafka Python package.
This package is licensed under the Apache 2.0 License.
"""
from __future__ import annotations

# standard library
from typing import ClassVar

from .cimpl import KafkaError as KafkaError
from .cimpl import KafkaException as KafkaException
from .cimpl import Message
from .serialization import SerializationError as SerializationError

class _KafkaClientError(KafkaException):
    """
    Wraps all errors encountered by a Kafka Client

    Args:
        kafka_error (KafkaError): KafkaError instance.

        exception(Exception, optional): The original exception

        kafka_message (Message, optional): The Kafka Message returned
        by the broker.
    """

    exception: ClassVar[KafkaException]
    kafka_message: ClassVar[Message]

    def __init__(
        self,
        kafka_error: KafkaError,
        exception: Exception | None = None,
        kafka_message: Message | None = None,
    ) -> None: ...
    @property
    def code(self): ...
    @property
    def name(self): ...

class ConsumeError(_KafkaClientError):
    """
    Wraps all errors encountered during the consumption of a message.

    Note:
        In the event of a serialization error the original message
        contents may be retrieved from the ``kafka_message`` attribute.

    Args:
        kafka_error (KafkaError): KafkaError instance.

        exception(Exception, optional): The original exception

        kafka_message (Message, optional): The Kafka Message
        returned by the broker.

    """

    def __init__(
        self,
        kafka_error: KafkaError,
        exception: Exception | None = None,
        kafka_message: Message | None = None,
    ) -> None: ...

class KeyDeserializationError(ConsumeError, SerializationError):
    def __init__(self, exception: Exception | None = None, kafka_message: Message | None = None) -> None: ...

class ValueDeserializationError(ConsumeError, SerializationError):
    def __init__(self, exception: Exception | None = None, kafka_message: Message | None = None) -> None: ...

class ProduceError(_KafkaClientError):
    def __init__(
        self,
        kafka_error: KafkaError,
        exception: Exception | None = None,
    ) -> None: ...

class KeySerializationError(ProduceError, SerializationError):
    def __init__(self, exception: Exception | None = None) -> None: ...

class ValueSerializationError(ProduceError, SerializationError):
    def __init__(self, exception: Exception | None = None) -> None: ...
